﻿<#
Prequisites
    - Install Waldo's Cloud Ready Software PowerShell modules 
        https://github.com/waldo1001/Cloud.Ready.Software.PowerShell
    - The variables from the script called "1 - set up environment.ps1" are all loaded into memory

This script will:
    - step 1: Create the folder structure for the object files
    - step 2: Extract all objects from the ORIGINAL environment and split into individual files
    - step 3: Extract all objects from the MODIFIED environment and split into individual files
    - step 4: Extract all objects from the TARGET environment and split into individual files
    - step 5: Merge the ORIGINAL, MODIFIED and TARGET objects into the RESULT folder
    - step 6: Merge the Version List values
    - step 7: Join the merged objects into a single object file called RESULTS.TXT

Here is how I used this script to localize an ISV product from W1 into NA:
1 - Copy all product specific objects from the W1 product database to the NA product database
      so I did not have to import those as text files and re-compile them
2 - Run steps 1-4
3 - Use a text compare tool to eliminate all identical objects, and all product specific objects
      I am only interested in merging the modified objects
4 - Run step 5 and 6 of the script - this will create the content of the RESULT folder:
      * individual merged object files, including successfully merged and failed ones
      * all with merged Version List values
      * CONFLICT files that describe the merge conflict
5 - Manually resolve the merge conflicts, using Visual Studio Code with a split window
      on the left with the CONFLICT file, on the right with the object file
      Most merge conflicts can be resolved in VSCode
6 - Run step 7 of the script, which creates a single text file that includes all merged objects
7 - Import the RESULTS.TXT file into the RESULT environment. Compile all uncompiled objects and
      resolve any remaining conflicts.

NOTE: This script is provided "AS IS" without any guarantees, warranties, or support

#>

# Step 1 - create the object folders if they don't already exist. 
#            In this case a folder called "Objects" in the working folder
#            plus folders called ORIGINAL, MODIFIED, TARGET, and RESULT
$ObjectsFolder = Join-Path $WorkingFolder Objects
$OriginalFolder = Join-Path $ObjectsFolder ORIGINAL
$ModifiedFolder = Join-Path $ObjectsFolder MODIFIED
$TargetFolder = Join-Path $ObjectsFolder TARGET
$ResultsFolder = Join-Path $ObjectsFolder RESULTS
if (-not(Test-Path $ObjectsFolder)) { New-Item -ItemType Directory -Path $ObjectsFolder}
if (-not(Test-Path $OriginalFolder)) {New-Item -ItemType Directory -Path $OriginalFolder}
if (-not(Test-Path $ModifiedFolder)) {New-Item -ItemType Directory -Path $ModifiedFolder}
if (-not(Test-Path $TargetFolder)) {New-Item -ItemType Directory -Path $TargetFolder}
if (-not(Test-Path $ResultsFolder)) {New-Item -ItemType Directory -Path $ResultsFolder}


# Step 2 - Export objects from ORIGINAL, split into individual object files
Export-NAVApplicationObject -DatabaseServer $DBServer `
                            -DatabaseName $EnvOriginal `
                            -Path (Join-Path $ObjectsFolder ORIGINAL.txt) |
    Split-NAVApplicationObjectFile -Destination $OriginalFolder -Force

# Step 3 - Export objects from MODIFIED, split into individual object files
Export-NAVApplicationObject -DatabaseServer $DBServer `
                            -DatabaseName $EnvModified `
                            -ExportTxtSkipUnlicensed `
                            -Path (Join-Path $ObjectsFolder MODIFIED.txt) |
    Split-NAVApplicationObjectFile -Destination $ModifiedFolder -Force

# Step 4 - Export objects from TARGET, split into individual object files
Export-NAVApplicationObject -DatabaseServer $DBServer `
                            -DatabaseName $EnvTarget `
                            -ExportTxtSkipUnlicensed `
                            -Path (Join-Path $ObjectsFolder RESULT.txt) |
    Split-NAVApplicationObjectFile -Destination $ResultsFolder -Force

# Use a text compare tool to eliminate everything except modified object files

# Step 5 - merge ORIGINAL, MODIFIED and TARGET objects into the RESULTS folder
$Mergeresult = Merge-NAVApplicationObject -OriginalPath $OriginalFolder `
                           -ModifiedPath $ModifiedFolder `
                           -TargetPath $TargetFolder `
                           -ResultPath $ResultsFolder `
                           -VersionListProperty FromTarget `
                           -DateTimeProperty FromTarget 

# Step 6 - merge version list values from the 3 environments
#   gets the highest version number for the predefined prefixes, 
#   and copies everything else
$Mergeresult |
    Where-Object {$_.MergeResult –eq 'Merged' -or $_.MergeResult –eq 'Conflict'}  |  
        Merge-NAVApplicationObjectProperty -UpdateDateTime $false `
                                           -UpdateVersionList $true `
                                           -VersionListPrefixes 'NAVW1','NAVNA' `
                                           -SwitchOriginalDate:$false `
                                           -SwitchModifiedDate:$false `
                                           -SwitchTargetDate:$false `
                                           -verbose
    

# Use Visual Studio Code to resolve the conflicts in the RESULTS folder

# Step 7 - Combine the object files in the RESULTS folder into a single object file
Join-NAVApplicationObjectFile -Source (Join-Path $ResultsFolder *.txt) `
                              -Destination (Join-Path $ObjectsFolder RESULTS.TXT)

# Manually import RESULTS.TXT into the RESULT environment
# Manually compile and resolve any remaining conflicts there
# Manually create a backup file and/or a full object extract for delivery
